
// =======================================================
// ++++++++++++++++++++++ FUNCIONES ++++++++++++++++++++++
// =======================================================

// -------------------------------------------------------
// Funcion que lee el pin analogico conectado al
// divisor de tensiones para medir la carga de la bateria
// el valor leido lo convierte al rango de 0 a 4.2V
// -------------------------------------------------------
void LeeBateria()
{
  static double in_max = 2435.00;
  static double in_min = 0;
  static double out_max = 4.2;
  static double out_min = 0;
  static double lectura = 0;

  lectura = analogRead(PinAnalog);
  delay(100);
  double carga = (lectura - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
  if (carga >= 4)
  {
    Bateria = completa;
  }
  else if (carga >= 3.8)
  {
    Bateria = mediaAlta;
  }
  else if (carga >= 3.6)
  {
    Bateria = mediaBaja;
  }
  else if (carga < 3.6)
  {
    Bateria = baja;
  }
}

// -------------------------------------------------------
// Funcion que consulta las paginas de datos del servidor
// Obtiene un String, que se "limpia" para eliminar posibles
// caracteres no imprimibles
// -------------------------------------------------------
String httpGETRequest(const char* serverName) {
  HTTPClient http;

  http.begin(serverName);

  // Send HTTP POST request
  int HTTPCodigoRespuesta = http.GET();

  String contenido = "";
  do
  {
    if (HTTPCodigoRespuesta > 0) {
      contenido = http.getString();
    }
    else {
      HTTPCodigoRespuesta = http.GET();
      delay(100);
    }

  } while (HTTPCodigoRespuesta <= 0);
  // Free resources
  http.end();
  //contenido.trim();
  return contenido;
}

// -------------------------------------------------------
// Funcion que sincroniza el tiempo del visualizador con
// el del servidor.
// Realiza consultas de la hora cada 5s, de manera que cuando
// el valor anterior y el nuevo son distintos, el servidor
// acaba de obtener nuevos datos, quedando sincronizado el
// intervalo de 60s con la disponibilidad de nuevos valores.
// -------------------------------------------------------
void SincronizaConServidor()
{
  String HoraAnt, HoraActual;
  tft.fillScreen(TFT_WHITE);
  drawBmp("/Sincronizando.bmp", 80, 36);
  HoraAnt = httpGETRequest(ServidorHora);
  HoraActual = HoraAnt;
  while (HoraActual == HoraAnt)
  {
    delay(5000);
    if (WiFi.status() == WL_CONNECTED ) {
      HoraActual = httpGETRequest(ServidorHora);
    }
    else
    {

      tft.fillScreen(TFT_WHITE);
      drawBmp("/Conectando.bmp", 91, 49);
      while (WiFi.status() != WL_CONNECTED) {
        delay(500);
      }

      tft.fillScreen(TFT_WHITE);
      drawBmp("/Sincronizando.bmp", 80, 36);
      HoraAnt = httpGETRequest(ServidorHora);
    }
  }
}

// -------------------------------------------------------
// Funcion que consulta los valores al servidor
// En funcion de la zona activa accede a una información
// u otra
// Además, como los valores de T y H del servidor utilizan
// como separador decimal "," y la funcion toDouble()
// necesita "." se reemplaza la "," por "."
// -------------------------------------------------------
void ConsultaDatos ()
{
  datosZona.Fecha = httpGETRequest(ServidorFecha);
  datosZona.Hora = httpGETRequest(ServidorHora);
  String str;
  if (parametros.Zona == "Calibracion")
  {
    str = httpGETRequest(ServidorTempCalibracion);
    str.replace(",", ".");
    datosZona.Temperatura = str.toDouble();

    str = httpGETRequest(ServidorHumCalibracion);
    str.replace(",", ".");
    datosZona.Humedad = str.toDouble();
  }
  else if (parametros.Zona == "Ensayos")
  {
    str = httpGETRequest(ServidorTempEnsayos);
    str.replace(",", ".");
    datosZona.Temperatura = str.toDouble();

    str = httpGETRequest(ServidorHumEnsayos);
    str.replace(",", ".");
    datosZona.Humedad = str.toDouble();
  }
  EstableceNivelDeAlerta ();
}

// -------------------------------------------------------
// Como memoria permanente se utiliza el sistema SPIFFS
// Los parametros configurados por el usuario se almacenan
// en un fichero de texto en lineas.
// Esta funcion lee cada linea del fichero y la convierte
// al tipo de variable necesaria
// -------------------------------------------------------
void LeeFicheroConfiguracion ()
{
  File FichConf = SPIFFS.open("/ParametrosConf.txt", "r");
  if (!FichConf)
  {
    //Serial.println("Fallo al abrir el archivo para leer");
  }
  else
  {
    parametros.Zona = FichConf.readStringUntil('\n');
    parametros.Zona.trim();
    parametros.TinfEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.TsupEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.TinfCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.TsupCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.HsupEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.HsupCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.HinfEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.HinfCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.IncerTempEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.IncerTempCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.IncerHumEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.IncerHumCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.BandaAmaTEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.BandaAmaTCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.BandaAmaHEnsayos = FichConf.readStringUntil('\n').toDouble();
    parametros.BandaAmaHCalibracion = FichConf.readStringUntil('\n').toDouble();
    parametros.TBloqueo = atol(FichConf.readStringUntil('\n').c_str());
    delay(100);
    FichConf.close();
    delay(100);
  }
}

// -------------------------------------------------------
// Como memoria permanente se utiliza el sistema SPIFFS
// Los parametros configurados por el usuario se almacenan
// en un fichero de texto en lineas.
// -------------------------------------------------------
void EscribeFicheroConfiguracion ()
{
  File FichConf = SPIFFS.open("/ParametrosConf.txt", "w");
  if (!FichConf)
  {
    //Serial.println("Fallo al abrir el archivo para escribir");
  }
  else
  {
    FichConf.println(parametros.Zona);
    FichConf.println(String(parametros.TinfEnsayos ));
    FichConf.println(String( parametros.TsupEnsayos));
    FichConf.println( String(parametros.TinfCalibracion));
    FichConf.println( String(parametros.TsupCalibracion));
    FichConf.println(String( parametros.HsupEnsayos));
    FichConf.println(String( parametros.HsupCalibracion));
    FichConf.println(String( parametros.HinfEnsayos));
    FichConf.println(String( parametros.HinfCalibracion));
    FichConf.println( String( parametros.IncerTempEnsayos));
    FichConf.println(String( parametros.IncerTempCalibracion));
    FichConf.println( String( parametros.IncerHumEnsayos));
    FichConf.println(String( parametros.IncerHumCalibracion));
    FichConf.println(String( parametros.BandaAmaTEnsayos));
    FichConf.println(String( parametros.BandaAmaTCalibracion));
    FichConf.println(String( parametros.BandaAmaHEnsayos));
    FichConf.println(String( parametros.BandaAmaHCalibracion));
    FichConf.print( String( parametros.TBloqueo));
    delay(100);
    FichConf.close();
    delay(100);
  }
}

// -------------------------------------------------------
// Al cambiar de zona se carga la configuracion guardada
// De memoria al dispositivo
// -------------------------------------------------------
void ActualizaParametrosCliente()
{
  if (parametros.Zona == "Calibracion")
  {
    datosZona.TemperaturaMax = parametros.TsupCalibracion;
    datosZona.TemperaturaMin = parametros.TinfCalibracion;
    datosZona.HumedadMax = parametros.HsupCalibracion;
    datosZona.IncerTemp = parametros.IncerTempCalibracion;
    datosZona.IncerHum = parametros.IncerHumCalibracion;
    datosZona.BandaAmaT = parametros.BandaAmaTCalibracion;
    datosZona.BandaAmaH = parametros.BandaAmaHCalibracion;
    datosZona.HumedadMin = parametros.HinfCalibracion;
  }
  else if (parametros.Zona == "Ensayos")
  {
    datosZona.TemperaturaMax = parametros.TsupEnsayos;
    datosZona.TemperaturaMin = parametros.TinfEnsayos;
    datosZona.HumedadMax = parametros.HsupEnsayos;
    datosZona.IncerTemp = parametros.IncerTempEnsayos;
    datosZona.IncerHum = parametros.IncerHumEnsayos;
    datosZona.BandaAmaT = parametros.BandaAmaTEnsayos;
    datosZona.BandaAmaH = parametros.BandaAmaHEnsayos;
    datosZona.HumedadMin = parametros.HinfEnsayos;
  }
}

// -------------------------------------------------------
// Al cambiar la configuracion se almacena
// Del dispositivo a memoria
// -------------------------------------------------------
void ActualizaParametrosFichero()
{
  if (parametros.Zona == "Calibracion")
  {
    parametros.TsupCalibracion = datosZona.TemperaturaMax;
    parametros.TinfCalibracion = datosZona.TemperaturaMin;
    parametros.HsupCalibracion = datosZona.HumedadMax;
    parametros.IncerTempCalibracion = datosZona.IncerTemp;
    parametros.IncerHumCalibracion = datosZona.IncerHum;
    parametros.BandaAmaTCalibracion = datosZona.BandaAmaT;
    parametros.BandaAmaHCalibracion = datosZona.BandaAmaH;
    parametros.HinfCalibracion = datosZona.HumedadMin;
  }
  else if (parametros.Zona == "Ensayos")
  {
    parametros.TsupEnsayos = datosZona.TemperaturaMax;
    parametros.TinfEnsayos = datosZona.TemperaturaMin;
    parametros.HsupEnsayos = datosZona.HumedadMax ;
    parametros.IncerTempEnsayos = datosZona.IncerTemp;
    parametros.IncerHumEnsayos = datosZona.IncerHum;
    parametros.BandaAmaTEnsayos = datosZona.BandaAmaT;
    parametros.BandaAmaHEnsayos = datosZona.BandaAmaH;
    parametros.HinfEnsayos = datosZona.HumedadMin;
  }
}

// -------------------------------------------------------
// En funcion de la configuracion de T y H y de los valores
// recibidos se establece el nivel de alerta
// -------------------------------------------------------
void EstableceNivelDeAlerta ()
{
  if ((datosZona.Humedad + datosZona.IncerHum)  < (datosZona.HumedadMax - datosZona.BandaAmaH) &&
      (datosZona.Humedad - datosZona.IncerHum)  > (datosZona.HumedadMin + datosZona.BandaAmaH))
  {
    datosZona.NivelDeAlertaHum = verde;
  }
  else if (((datosZona.Humedad + datosZona.IncerHum) >= datosZona.HumedadMax) ||
           ((datosZona.Humedad - datosZona.IncerHum) <= datosZona.HumedadMin))
  {
    datosZona.NivelDeAlertaHum = rojo;
  }
  else
  {
    datosZona.NivelDeAlertaHum = amarillo;
  }

  if (((datosZona.Temperatura - datosZona.IncerTemp) > (datosZona.TemperaturaMin + datosZona.BandaAmaT)) &&
      ((datosZona.Temperatura + datosZona.IncerTemp) < datosZona.TemperaturaMax - datosZona.BandaAmaT))
  {
    datosZona.NivelDeAlertaTemp = verde;
  }
  else if (((datosZona.Temperatura - datosZona.IncerTemp) <= datosZona.TemperaturaMin) ||
           ((datosZona.Temperatura + datosZona.IncerTemp ) >= datosZona.TemperaturaMax))
  {
    datosZona.NivelDeAlertaTemp = rojo;
  }
  else
  {
    datosZona.NivelDeAlertaTemp = amarillo;
  }

  if (((datosZona.NivelDeAlertaTemp == rojo ) || (datosZona.NivelDeAlertaHum == rojo)) )
  {
    if (activarAlerta && (menuActivo == bloqueo || menuActivo == principal))
    {
      menuActivo = alerta;
      activarAlerta = false;
      actualizar = true;
    }
  }
  else
  {
    activarAlerta = true;
    if (menuActivo == alerta)
    {
      menuActivo = principal;
      ledcWrite(BuzzChannel, 0);
    }
  }
}


// -------------------------------------------------------
// sonido al tocar en la pantalla
// el usuario puede desactivarlo
// -------------------------------------------------------
void Pitido ()
{
  if (SonidoOn)
  {
    ledcWrite(BuzzChannel, frecPitido);
    delay(200);
    ledcWrite(BuzzChannel, 0);
  }
}

