
// =======================================================
// ++++++++++++++++++++++++ MENUS ++++++++++++++++++++++++
// =======================================================

// ========================================================
// A continuacion se programan las interfaces de los menus
// Todos ellos siguen la misma estructura
// Si se muestran por primera vez (actualizar = true):
// se cargan las imágenes que forman la interfaz
// De manera continua se comprueba si se ha dado una
// pulsacion sobre la pantalla. Si es así, en función
// de las coordenadas del toque se cambia de menu
// ========================================================

// -------------------------------------------------------
// Menu de la pantalla de bloqueo
// - Fecha y hora
// - Icono con el nivel de carga de la bateria
// - Imagen de un semaforo con el color del nivel de alarma
// - Simbolo de tick, exclamacion o cruz en colores según el
//   el nivel de alarma
// -------------------------------------------------------
void menuBloqueo()
{

  if (actualizar == true )
  {
    actualizar = false;
    LeeBateria();
    tft.fillScreen(TFT_WHITE);
    tft.drawCentreString(String(datosZona.Fecha), 160, 10, 2);
    tft.drawCentreString(String(datosZona.Hora), 160, 35, 2);

    // Se muestran distintas imagenes en funcion del nivel de alerta
    if (datosZona.NivelDeAlertaTemp == verde && datosZona.NivelDeAlertaHum == verde)
    {
      drawBmp("/SemaforoVerde.bmp", 135, 80);
      drawBmp("/Tick.bmp", 32, 115);
      drawBmp("/Tick.bmp", 217, 115);
    }
    else if ((datosZona.NivelDeAlertaTemp == amarillo && datosZona.NivelDeAlertaHum != rojo) ||
             (datosZona.NivelDeAlertaTemp != rojo && datosZona.NivelDeAlertaHum == amarillo))
    {
      drawBmp("/SemaforoAmarillo.bmp", 135, 80);
      drawBmp("/Exclamacion.bmp", 32, 115);
      drawBmp("/Exclamacion.bmp", 217, 115);
    }
    else
    {
      drawBmp("/SemaforoRojo.bmp", 135, 80);
      drawBmp("/Cruz.bmp", 32, 115);
      drawBmp("/Cruz.bmp", 217, 115);
    }
    // Se muestra el nivel de carga de la bateria
    switch (Bateria)
    {
      case completa:
        drawBmp("/CargaCompleta.bmp", 258, 10);
        break;
      case mediaAlta:
        drawBmp("/CargaMediaAlta.bmp", 258, 10);
        break;
      case mediaBaja:
        drawBmp("/CargaMediaBaja.bmp", 258, 10);
        break;
      case baja:
        drawBmp("/CargaBaja.bmp", 258, 10);
        break;
    }
    delay(100);
  }
  // se desbloquea -> se pasa al menu principal
  if (tft.getTouch(&x, &y))
  {
    Pitido();
    menuActivo = principal;
    actualizar = true;
    t_bloqueo = currentMillis;
  }
}

// -------------------------------------------------------
// Menu de la pantalla principal
// - Zona
// - Temperatura y humedad de la zona
// - Simbolos de temperatura y humedad de distintos colores
//   segun el nivel de alerta
// - Simbolo de engranaje para acceso al menu de configuracion
// -------------------------------------------------------
void menuPrincipal()
{
  if (actualizar == true )
  {

    actualizar = false;
    tft.setTextSize(6);
    tft.setCursor(0, 0);
    tft.fillScreen(TFT_WHITE);
    delay(100);
    drawBmp("/Conf.bmp", 270, 190);
    ////Alineacion centrada
    tft.setTextDatum(MC_DATUM);
    tft.drawString(String(parametros.Zona), 160, 20, 5);
    tft.drawString(String(datosZona.Hora) + "  " + String(datosZona.Fecha) , 160, 55, 5);

    tft.setTextColor(TFT_BLACK, TFT_WHITE);
    //Alineacion centrada izquierda
    tft.setTextDatum(ML_DATUM);
    tft.drawString(String(datosZona.Temperatura), 140, 110, 6);
    tft.drawString(String(datosZona.Humedad), 140, 190, 6);
    switch (datosZona.NivelDeAlertaHum)
    {
      case verde:
        drawBmp("/HumVerde.bmp", 76, 152);
        break;
      case amarillo:
        drawBmp("/HumNaranja.bmp", 76, 152);
        break;
      case rojo:
        drawBmp("/HumRojo.bmp", 76, 152);
        break;
    }
    switch (datosZona.NivelDeAlertaTemp)
    {
      case verde:
        drawBmp("/TempVerde.bmp", 76, 72);
        break;
      case amarillo:
        drawBmp("/TempNaranja.bmp", 76, 72);
        break;
      case rojo:
        drawBmp("/TempRojo.bmp", 76, 72);
        break;
    }

    LeeBateria();
    // Se muestran distintas imagenes en funcion del nivel de alerta
    if (datosZona.NivelDeAlertaTemp == verde && datosZona.NivelDeAlertaHum == verde)
    {
      drawBmp("/SemaforoVerde.bmp", 15, 80);
    }
    else if ((datosZona.NivelDeAlertaTemp == amarillo && datosZona.NivelDeAlertaHum != rojo) ||
             (datosZona.NivelDeAlertaTemp != rojo && datosZona.NivelDeAlertaHum == amarillo))
    {
      drawBmp("/SemaforoAmarillo.bmp", 15, 80);
    }
    else
    {
      drawBmp("/SemaforoRojo.bmp", 15, 80);
    }
    // Se muestra el nivel de carga de la bateria
    switch (Bateria)
    {
      case completa:
        drawBmp("/CargaCompleta.bmp", 258, 10);
        break;
      case mediaAlta:
        drawBmp("/CargaMediaAlta.bmp", 258, 10);
        break;
      case mediaBaja:
        drawBmp("/CargaMediaBaja.bmp", 258, 10);
        break;
      case baja:
        drawBmp("/CargaBaja.bmp", 258, 10);
        break;
    }

    //t_bloqueo = currentMillis;
    delay(500);
  }

  if (tft.getTouch(&x, &y))
  {
    if (x > 280 && x < 320) {
      if (y > 0 && y < 40) {
        Pitido();
        menuActivo = configuracion;
        actualizar = true;
      }
    }
  }
}

// -------------------------------------------------------
// Menu de la pantalla de configuracion
// -Se muestran cuatro botones, que dan acceso a las
//  configuraciones específicas
// - Botón de salir para volver al menu principal
// -------------------------------------------------------
void menuConfiguracion()
{

  if (actualizar == true )
  {
    actualizar = false;
    EscribeFicheroConfiguracion ();
    EstableceNivelDeAlerta ();
    drawBmp("/MenuConfiguracion.bmp", 0, 0);
    if (SonidoOn)
    {
      drawBmp("/SonidoOn.bmp", 280, 10);
    }
    else
    {
      drawBmp("/SonidoOff.bmp", 280, 10);
    }
    delay(400);
  }


  if (tft.getTouch(&x, &y))
  {
    if (y > 210 && y < 240) {
      if (x > 0 && x < 30) {
        Pitido();
        menuActivo = principal;
        ConsultaDatos();
        actualizar = true;
        t_bloqueo = currentMillis;
      }
      else if (x > 284 && x < 310)
      {
        SonidoOn = !SonidoOn;

        if (SonidoOn)
        {
          drawBmp("/SonidoOn.bmp", 280, 10);
        }
        else
        {
          drawBmp("/SonidoOff.bmp", 280, 10);
        }
        delay(100);
      }
    }
    else if (y > 170 && y < 200)
    {
      if (x > 30 && x < 290) {
        Pitido();
        menuActivo = zonas;
        actualizar = true;
      }
    }
    else if (y > 120 && y < 150)
    {
      if (x > 30 && x < 290) {
        Pitido();
        menuActivo = limitesTemp;
        actualizar = true;
      }
    }
    else if (y > 70 && y < 100)
    {
      if (x > 30 && x < 290) {
        Pitido();
        menuActivo = limiteHum;
        actualizar = true;
      }
    }
    else if (y > 20 && y < 50)
    {
      if (x > 30 && x < 290) {
        Pitido();
        menuActivo = bandasAmarillas;
        actualizar = true;
      }
    }

    delay(100);
  }

}

// -------------------------------------------------------
// Menu de la pantalla de selección de zona
// - botones para elegir la zona activa
// - Flecha de salir para volver al menu de configuraciones
// -------------------------------------------------------
void menuSeleccZona()
{

  if (actualizar == true )
  {
    actualizar = false;
    drawBmp("/MenuZona.bmp", 0, 0);
    delay(500);
  }

  if (tft.getTouch(&x, &y))
  {
    if (y > 0 && y < 30) {
      if (x > 0 && x < 30) {
        Pitido();
        menuActivo = configuracion;
        actualizar = true;
      }
    }
    else if (y > 56 && y < 96)
    {
      if (x > 30 && x < 290) {
        menuActivo = configuracion;
        Pitido();
        parametros.Zona = "Ensayos";
        actualizar = true;
        ActualizaParametrosCliente();
        ConsultaDatos();
      }
    }
    else if (y > 144 && y < 184)
    {
      if (x > 30 && x < 290) {
        menuActivo = configuracion;
        Pitido();
        parametros.Zona = "Calibracion";
        actualizar = true;
        ActualizaParametrosCliente();
        ConsultaDatos();
      }
    }
  }
}

// -------------------------------------------------------
// Menu de la pantalla de conf de parametros de temperatura
// - Tres campos con + y - para cambiar el valor
//   + Temperatura máxima
//   + Incertidumbre temperatura
//   + Temperatura mínima
// - Flecha salir
// -------------------------------------------------------
void menuConfTemp()
{
  if (actualizar)
  {
    actualizar = false;
    actualizarTexto = true;
    tft.fillScreen(TFT_WHITE);
    tft.setTextSize(2);
    tft.drawCentreString("Tsup.      Ince.      Tinf.", 160, 10, 2);
    drawBmp("/CampoIncDec.bmp", 10, 35);
    drawBmp("/CampoIncDec.bmp", 115, 35);
    drawBmp("/CampoIncDec.bmp", 220, 35);
    drawBmp("/FlechaSalir.bmp", 0, 205);
    delay(100);

  }

  if (actualizarTexto)
  {
    actualizarTexto = false;
    tft.setTextColor(TFT_BLACK, TFT_WHITE);
    tft.drawCentreString(String(datosZona.TemperaturaMax), 55, 110, 2);
    tft.drawCentreString(String(datosZona.IncerTemp), 160, 110, 2);
    tft.drawCentreString(String(datosZona.TemperaturaMin), 265, 110, 2);
  }
  if (tft.getTouch(&x, &y))
  {
    if (y > 0 && y < 35) {
      if (x > 0 && x < 35) {
        menuActivo = configuracion;
        Pitido();
        ActualizaParametrosFichero();
        actualizar = true;
      }
    }
    else if (y > 170 && y < 200)
    {
      if (x > 40 && x < 70)
      {
        datosZona.TemperaturaMax += 0.1;
        if (datosZona.TemperaturaMax <= (datosZona.TemperaturaMin + 2))
        {
          datosZona.TemperaturaMax = datosZona.TemperaturaMin + 2.0;
        }
        drawBmp("/CampoIncDec.bmp", 10, 35);
        actualizarTexto = true;
        ActualizaParametrosFichero();
        delay(50);
      }
      else if (x > 145 && x < 175)
      {
        datosZona.IncerTemp += 0.1;
        drawBmp("/CampoIncDec.bmp", 115, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
      else if (x > 250 && x < 280)
      {
        datosZona.TemperaturaMin += 0.1;
        if (datosZona.TemperaturaMax <= (datosZona.TemperaturaMin + 2))
        {
          datosZona.TemperaturaMax = datosZona.TemperaturaMin + 2.0;
        }
        drawBmp("/CampoIncDec.bmp", 220, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
    }
    else if (y > 40 && y < 70)
    {
      if (x > 40 && x < 70)
      {
        datosZona.TemperaturaMax -= 0.1;
        if (datosZona.TemperaturaMax <= (datosZona.TemperaturaMin + 2))
        {
          datosZona.TemperaturaMax = datosZona.TemperaturaMin + 2.0;
        }
        drawBmp("/CampoIncDec.bmp", 10, 35);
        actualizarTexto = true;
        ActualizaParametrosFichero();
        delay(50);
      }
      else if (x > 145 && x < 175)
      {
        datosZona.IncerTemp -= 0.1;
        if (datosZona.IncerTemp <= 0)
        {
          datosZona.IncerTemp = 0;
        }
        drawBmp("/CampoIncDec.bmp", 115, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
      else if (x > 250 && x < 280)
      {
        datosZona.TemperaturaMin -= 0.1;
        if (datosZona.TemperaturaMin < 0)
        {
          datosZona.TemperaturaMin = 0;
        }
        if (datosZona.TemperaturaMax <= datosZona.TemperaturaMin)
        {
          datosZona.TemperaturaMax = datosZona.TemperaturaMin + 1.0;
        }
        drawBmp("/CampoIncDec.bmp", 220, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
    }
  }
}

// -------------------------------------------------------
// Menu de la pantalla de conf de parametros de humedad
// - Tres campos con + y - para cambiar el valor
//   + Humedad máxima
//   + Incertidumbre humedad
//   + Humedad minima
// - Flecha salir
// -------------------------------------------------------
void menuConfhum()
{
  if (actualizar)
  {
    actualizar = false;
    actualizarTexto = true;
    tft.fillScreen(TFT_WHITE);
    tft.drawCentreString("Hsup.      Ince.      Hinf.", 160, 10, 2);
    drawBmp("/CampoIncDec.bmp", 10, 35);
    drawBmp("/CampoIncDec.bmp", 115, 35);
    drawBmp("/CampoIncDec.bmp", 220, 35);
    drawBmp("/FlechaSalir.bmp", 0, 205);
    delay(100);
  }

  if (actualizarTexto)
  {
    actualizarTexto = false;
    tft.setTextColor(TFT_BLACK, TFT_WHITE);
    tft.drawCentreString(String(datosZona.HumedadMax), 55, 110, 2);
    tft.drawCentreString(String(datosZona.IncerHum), 160, 110, 2);
    tft.drawCentreString(String(datosZona.HumedadMin), 265, 110, 2);
  }

  if (tft.getTouch(&x, &y))
  {
    if (y > 0 && y < 35) {
      if (x > 0 && x < 35) {
        Pitido();
        menuActivo = configuracion;
        ActualizaParametrosFichero();
        actualizar = true;
      }
    }
    else if (y > 170 && y < 200)
    {
      if (x > 40 && x < 70)
      {
        datosZona.HumedadMax += 0.5;
        if (datosZona.HumedadMax > 100)
        {
          datosZona.HumedadMax = 100;
        }
        actualizarTexto = true;
        drawBmp("/CampoIncDec.bmp", 10, 35);
        ActualizaParametrosFichero();
        delay(50);
      }
      else if (x > 145 && x < 175)
      {
        datosZona.IncerHum += 0.1;
        drawBmp("/CampoIncDec.bmp", 115, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
      else if (x > 250 && x < 280)
      {
        datosZona.HumedadMin += 0.5;
        if ( datosZona.HumedadMin > 95)
        {
          datosZona.HumedadMin = 95;
        }
        if (datosZona.HumedadMax <= datosZona.HumedadMin)
        {
          datosZona.HumedadMax = datosZona.HumedadMin + 5.0;
        }

        drawBmp("/CampoIncDec.bmp", 220, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
    }
    else if (y > 40 && y < 70)
    {
      if (x > 40 && x < 70)
      {
        datosZona.HumedadMax -= 0.5;
        if (datosZona.HumedadMax < 0)
        {
          datosZona.HumedadMax = 0;
        }
        if (datosZona.TemperaturaMax <= datosZona.TemperaturaMin)
        {
          datosZona.TemperaturaMax = datosZona.TemperaturaMin + 5;
        }

        drawBmp("/CampoIncDec.bmp", 10, 35);
        actualizarTexto = true;
        ActualizaParametrosFichero();
        delay(50);
      }
      else if (x > 145 && x < 175)
      {
        datosZona.IncerHum -= 0.1;
        if (datosZona.IncerHum <= 0)
        {
          datosZona.IncerHum = 0;
        }
        drawBmp("/CampoIncDec.bmp", 115, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
      else if (x > 250 && x < 280)
      {
        datosZona.HumedadMin -= 0.5;
        if ( datosZona.HumedadMin < 0)
        {
          datosZona.HumedadMin = 0;
        }

        drawBmp("/CampoIncDec.bmp", 220, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
    }
  }
}

// -------------------------------------------------------
// Menu de la pantalla de conf de parametros de humedad
// - Dos campos con + y - para cambiar el valor
//   + Banda amarilla de temperatura
//   + Banda amarilal de humedad
// - Flecha salir
// -------------------------------------------------------
void menuBandasAmarillas()
{
  if (actualizar)
  {
    actualizar = false;
    actualizarTexto = true;
    tft.fillScreen(TFT_WHITE);
    tft.drawCentreString("BandaT.", 75, 10, 2);
    tft.drawCentreString("BandaH.", 235, 10, 2);
    drawBmp("/CampoIncDec.bmp", 35, 35);
    drawBmp("/CampoIncDec.bmp", 195, 35);
    drawBmp("/FlechaSalir.bmp", 0, 205);
    delay(100);
  }

  if (actualizarTexto)
  {
    actualizarTexto = false;
    tft.setTextColor(TFT_BLACK, TFT_WHITE);
    tft.drawCentreString(String(datosZona.BandaAmaT), 80, 110, 2);
    tft.drawCentreString(String(datosZona.BandaAmaH), 240, 110, 2);
  }

  if (tft.getTouch(&x, &y))
  {
    if (y > 0 && y < 35) {
      if (x > 0 && x < 35) {
        Pitido();
        menuActivo = configuracion;
        ActualizaParametrosFichero();
        actualizar = true;
      }
    }
    else if (y > 170 && y < 200)
    {
      if (x > 65 && x < 95)
      {
        datosZona.BandaAmaT += 0.1;
        if (datosZona.BandaAmaT <= 0)
        {
          datosZona.BandaAmaT = 0;
        }
        actualizarTexto = true;
        drawBmp("/CampoIncDec.bmp", 35, 35);
        ActualizaParametrosFichero();
        delay(50);
      }
      else if (x > 225 && x < 255)
      {
        datosZona.BandaAmaH += 0.1;
        drawBmp("/CampoIncDec.bmp", 195, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
    }
    else if (y > 40 && y < 70)
    {
      if (x > 65 && x < 95)
      {
        datosZona.BandaAmaT -= 0.1;
        if (datosZona.BandaAmaT < 0)
        {
          datosZona.BandaAmaT = 0;
        }
        drawBmp("/CampoIncDec.bmp", 35, 35);
        actualizarTexto = true;
        ActualizaParametrosFichero();
        delay(50);
      }
      else if (x > 225 && x < 255)
      {
        datosZona.BandaAmaH -= 0.1;
        if (datosZona.BandaAmaH < 0)
        {
          datosZona.BandaAmaH = 0;
        }
        drawBmp("/CampoIncDec.bmp", 195, 35);
        ActualizaParametrosFichero();
        actualizarTexto = true;
        delay(50);
      }
    }
  }
}

/*
  // -------------------------------------------------------
  // Menu de la pantalla de configuracion del bloqueo
  // - Un campo con + y - para cambiar el valor del tiempo
  // de bloqueo
  // - Boton para activar o desactivar el sonido de pulsacion
  // -------------------------------------------------------
  void menuConfTBloqueo()
  {
  if (actualizar)
  {
    actualizar = false;
    actualizarTexto = true;
    tft.fillScreen(TFT_WHITE);
    drawBmp("/CampoIncDec.bmp", 35, 35);
    if (SonidoOn)
    {
      drawBmp("/SonidoOn.bmp", 210, 90);
    }
    else
    {
      drawBmp("/SonidoOff.bmp", 210, 90);
    }
    tft.drawCentreString("TBloqueo", 80, 10, 2);
    drawBmp("/FlechaSalir.bmp", 0, 205);
    delay(100);
  }
  if (actualizarTexto)
  {
    actualizarTexto = false;
    tft.setTextColor(TFT_BLACK, TFT_WHITE);

    tft.drawCentreString(" " + String(+parametros.TBloqueo / 1000) + "s ", 80, 110, 2);
  }

  if (tft.getTouch(&x, &y))
  {
    if (y > 0 && y < 35) {
      if (x > 0 && x < 35) {
        Pitido();
        menuActivo = configuracion;
        ActualizaParametrosFichero();
        actualizar = true;
      }
    }
    else if (y > 170 && y < 200)
    {
      if (x > 65 && x < 95)
      {
        parametros.TBloqueo += 500;
        if (parametros.TBloqueo >= 60000)
        {
          parametros.TBloqueo = 60000;
        }
        actualizarTexto = true;
        ActualizaParametrosFichero();
        delay(50);
      }
    }
    else if (y > 40 && y < 70)
    {
      if (x > 65 && x < 95)
      {
        parametros.TBloqueo -= 500;
        if (parametros.TBloqueo <= 5000)
        {
          parametros.TBloqueo = 5000;
        }
        actualizarTexto = true;
        ActualizaParametrosFichero();
        delay(50);
      }
    }
    else if (y > 90 && y < 150)
    {
      if (x > 210 && x < 270)
      {
        SonidoOn = !SonidoOn;

        if (SonidoOn)
        {
          drawBmp("/SonidoOn.bmp", 210, 90);
        }
        else
        {
          drawBmp("/SonidoOff.bmp", 210, 90);
        }
        delay(100);
      }
    }
  }
  }
*/

// -------------------------------------------------------
// Pantalla en rojo con simbolo de advertencia
// Tambien activa el zumbador, pero solamente cuando salta
// la alerta, si el usuario toca la pantalla se desactiva
// y no se vuelve a activar hasta que se pase de
// verde/amarillo a rojo
// -------------------------------------------------------
void PantallaAlerta()
{
  if (actualizar)
  {
    actualizar = false;
    tft.fillScreen(TFT_RED);
    drawBmp("/Alerta.bmp", 85, 45);
    ledcWrite(BuzzChannel, 1500);
    tft.setTextDatum(MC_DATUM);
    tft.setTextColor(TFT_BLACK, TFT_RED);
    if (datosZona.NivelDeAlertaTemp == rojo)
    {
      if (datosZona.Temperatura <= datosZona.TemperaturaMin)
        tft.drawString("Temp. baja: " + String(datosZona.Temperatura)+ "C", 160, 30, 2);
      else
        tft.drawString("Temp. alta: " + String(datosZona.Temperatura)+ "C", 160, 30, 2);
    }
    if (datosZona.NivelDeAlertaHum == rojo)
    {
      if (datosZona.Humedad <= datosZona.HumedadMin)
        tft.drawString("Hum. baja: " + String(datosZona.Humedad) + "%", 160, 210, 2);
      else
        tft.drawString("Hum. alta: " + String(datosZona.Humedad)+ "%", 160, 210, 2);
    }


  }
  if (tft.getTouch(&x, &y))
  {
    ledcWrite(BuzzChannel, 0);
    menuActivo = principal;
    actualizar = true;
    tft.setTextColor(TFT_BLACK, TFT_WHITE);
  }
}
