
#include "librerias.h"
#define FS_NO_GLOBALS
// =======================================================
// +++++++++++++++++ VARIABLES GLOBALES ++++++++++++++++++
// =======================================================
// Pines y parametros PWM
// Los pines usados por la pantalla están definidos en el
// fichero "user_setup.h" de la libreria TFT_eSPI
//#define TFT_MISO 19
//#define TFT_MOSI 23
//#define TFT_SCLK 18
//#define TFT_CS   15  // Chip select control pin
//#define TFT_DC    4  // Data Command control pin
//#define TFT_RST   2  // Reset pin (could connect to RST pin)
//#define TOUCH_CS 21     // Chip select pin (T_CS) of touch screen

const int pinBuzz = 32;
const int freq = 2000;
const int BuzzChannel = 0;
const int resolution = 10;
const double frecPitido = 1000;
const int PinAnalog = 35;
// Uso de la clase TFT_eSPI
TFT_eSPI tft = TFT_eSPI();
// Ancho del intervalo amarillo para T y H
const double ZonaAmarillaTemp = 1.0; //1ºC de intervalo amarillo a cada lado
const double ZonaAmarillaHum = 5.0; //5% de intervalo amarillo
const double HumedadMinima = 20;
// periodo de consulta del servidor
const long intervalo = 60000;
// ++++++++++++++++++++++++++++++++++++++
// ID y contraseña del WiFi del servidor
char* ssid = "LACECAL_TyH";
const char* password = "TFG_LACECAL";
// ++++++++++++++++++++++++++++++++++++++

// flags
bool actualizar = true;
bool actualizarTexto = true;
bool SonidoOn = true;
bool activarAlerta = true;

// Coordenadas tactil
uint16_t x, y;
// variables para configurar los tiempos
unsigned long currentMillis = 0;
unsigned long previousMillis = 0;
unsigned long t_bloqueo = 0;


// Variables de tipo struct (ver "estructurasDatos.h")
ParametrosConf parametros;
DatosCliente datosZona;
Menus menuActivo;
NivelesDeCarga Bateria;

// -------------------------------------------------------
// Codigo que se ejecuta al encenderse el ESP32
// -------------------------------------------------------
void setup() {
  // pin de lectura de carga como entrada
  // y se establece una resolucion de 12bits para el ADC
  pinMode(PinAnalog, INPUT);
  analogReadResolution(12);
  
  // pin del zumbador como salida
  pinMode(pinBuzz, OUTPUT);
  // Se establece la frecuencia y resolucion de uno de los contadores
  // internos del ESP32
  // después se liga ese contador al pin conectado al buzzer
  ledcSetup(BuzzChannel, freq, resolution);
  ledcAttachPin(pinBuzz, BuzzChannel);

  // Inicio de SPIFFS para el acceso de los archivos
  if (!SPIFFS.begin()) {
    while (1) yield(); // Stay here twiddling thumbs waiting
  }

  // Conexion con la pantalla tft y configuracion
  // del tipo de fuente del texto y color
  tft.init();
  tft.setTextColor(TFT_BLACK);
  String fileName = "Final-Frontier-28";
  tft.setRotation(1);
  tft.loadFont(fileName);
  tft.setTextSize(4);
  tft.fillScreen(TFT_WHITE);
  // Se muestra la imagen de conectando
  drawBmp("/Conectando.bmp", 91, 49);
  // Se inicia el Wi-Fi
  WiFi.begin(ssid, password);
  // Se espera a la conexion con el servidor
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
  // Se leen y cargan los parámetros de la memoria
  LeeFicheroConfiguracion();
  ActualizaParametrosCliente();
  // Se sincroniza el tiempo del cliente con el del servidor
  if (WiFi.status() == WL_CONNECTED ) {
    SincronizaConServidor();
    ConsultaDatos();
  }
  
  // Se actualizan los temporizadores
  currentMillis = millis();
  previousMillis = currentMillis;
  t_bloqueo = currentMillis;
  menuActivo = principal;
}


// -------------------------------------------------------
// Codigo que se ejecuta de manera ciclica
// -------------------------------------------------------

void loop() {
  // tiempo actual
  currentMillis = millis();
  // cada minuto se consulta el servidor para obtener los nuevos datos
  if (currentMillis - previousMillis >= intervalo) {
    if (WiFi.status() == WL_CONNECTED ) {
      ConsultaDatos();
      // si los menus activos son el de bloqueo o el principal
      // se actualizan los valores
      // si es otro menu no se actualizan para no interrumpir
      // la configuracion por parte del usuario
      if (menuActivo == bloqueo || menuActivo == principal)
      {
        actualizar = true;
      }
    }
    // se pone a cero el contador para contar otro minuto
    previousMillis = currentMillis;
    t_bloqueo = currentMillis;
  }
  /*
    else
    {
    // si el menu mostrado es el principal y se cumple el tiempo de bloqueo
    // se muestra el menu de bloqueo
    if ((menuActivo == principal) && ((currentMillis - t_bloqueo) >= parametros.TBloqueo))
    {
      menuActivo = bloqueo;
      actualizar = true;
    }
    }
  */

  // Se muestran los distintos menus
  switch (menuActivo)
  {
//    case bloqueo:
//      menuBloqueo();
//      break;
    case principal:
      menuPrincipal();
      break;
    case configuracion:
      menuConfiguracion();
      break;
    case zonas:
      menuSeleccZona();
      break;
    case limitesTemp:
      menuConfTemp();
      break;
    case limiteHum:
      menuConfhum();
      break;
    case bandasAmarillas:
      menuBandasAmarillas();
      break;
    case alerta:
      PantallaAlerta();
      break;
    default:
      break;
  }

}



